// Copyright (C) 2020 ~ 2021 Uniontech Software Technology Co.,Ltd.
// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef CONSTANT_H
#define CONSTANT_H

class Constant
{
public:
    static const int RECTANGLE_PADDING;
    static const int RECTANGLE_RADIUS;
    static const int RECTANGLE_FONT_SIZE;
};

#endif
