/****************************************************************************
** Meta object code from reading C++ file 'tempfile.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "utils/tempfile.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tempfile.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8TempFileE_t {};
} // unnamed namespace

template <> constexpr inline auto TempFile::qt_create_metaobjectdata<qt_meta_tag_ZN8TempFileE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TempFile",
        "getFullscreenPixmap",
        "QPixmap",
        "",
        "getMosaicPixmap",
        "radius",
        "getBlurPixmap",
        "setFullScreenPixmap",
        "pixmap",
        "setBlurPixmap",
        "setMosaicPixmap",
        "isContainsPix",
        "effect"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'getFullscreenPixmap'
        QtMocHelpers::SlotData<const QPixmap() const>(1, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Slot 'getMosaicPixmap'
        QtMocHelpers::SlotData<const QPixmap(const int) const>(4, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'getBlurPixmap'
        QtMocHelpers::SlotData<const QPixmap(const int) const>(6, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'setFullScreenPixmap'
        QtMocHelpers::SlotData<void(const QPixmap &)>(7, 3, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 2, 8 },
        }}),
        // Slot 'setBlurPixmap'
        QtMocHelpers::SlotData<void(const QPixmap &, const int)>(9, 3, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 2, 8 }, { QMetaType::Int, 5 },
        }}),
        // Slot 'setMosaicPixmap'
        QtMocHelpers::SlotData<void(const QPixmap &, const int)>(10, 3, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 2, 8 }, { QMetaType::Int, 5 },
        }}),
        // Slot 'isContainsPix'
        QtMocHelpers::SlotData<bool(const QString, const int)>(11, 3, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 12 }, { QMetaType::Int, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TempFile, qt_meta_tag_ZN8TempFileE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TempFile::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TempFileE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TempFileE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8TempFileE_t>.metaTypes,
    nullptr
} };

void TempFile::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TempFile *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QPixmap _r = _t->getFullscreenPixmap();
            if (_a[0]) *reinterpret_cast<QPixmap*>(_a[0]) = std::move(_r); }  break;
        case 1: { QPixmap _r = _t->getMosaicPixmap((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QPixmap*>(_a[0]) = std::move(_r); }  break;
        case 2: { QPixmap _r = _t->getBlurPixmap((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QPixmap*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->setFullScreenPixmap((*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[1]))); break;
        case 4: _t->setBlurPixmap((*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->setMosaicPixmap((*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: { bool _r = _t->isContainsPix((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *TempFile::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TempFile::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TempFileE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TempFile::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
