/****************************************************************************
** Meta object code from reading C++ file 'voicevolumewatcher_interface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "utils/voicevolumewatcher_interface.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'voicevolumewatcher_interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN28voicevolumewatcher_interfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto voicevolumewatcher_interface::qt_create_metaobjectdata<qt_meta_tag_ZN28voicevolumewatcher_interfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "voicevolumewatcher_interface",
        "PortEnabledChanged",
        "",
        "in0",
        "in1",
        "in2",
        "IsPortEnabled",
        "QDBusPendingReply<bool>",
        "Reset",
        "QDBusPendingReply<>",
        "SetBluetoothAudioMode",
        "SetDefaultSink",
        "SetDefaultSource",
        "SetPort",
        "SetPortEnabled",
        "BluetoothAudioMode",
        "BluetoothAudioModeOpts",
        "Cards",
        "CardsWithoutUnavailable",
        "DefaultSink",
        "QDBusObjectPath",
        "DefaultSource",
        "IncreaseVolume",
        "MaxUIVolume",
        "ReduceNoise",
        "SinkInputs",
        "QList<QDBusObjectPath>",
        "Sinks",
        "Sources"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'PortEnabledChanged'
        QtMocHelpers::SignalData<void(uint, const QString &, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Slot 'IsPortEnabled'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(uint, const QString &)>(6, 2, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::UInt, 3 }, { QMetaType::QString, 4 },
        }}),
        // Slot 'Reset'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(8, 2, QMC::AccessPublic, 0x80000000 | 9),
        // Slot 'SetBluetoothAudioMode'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(10, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'SetDefaultSink'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(11, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'SetDefaultSource'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(12, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'SetPort'
        QtMocHelpers::SlotData<QDBusPendingReply<>(uint, const QString &, int)>(13, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::UInt, 3 }, { QMetaType::QString, 4 }, { QMetaType::Int, 5 },
        }}),
        // Slot 'SetPortEnabled'
        QtMocHelpers::SlotData<QDBusPendingReply<>(uint, const QString &, bool)>(14, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::UInt, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'BluetoothAudioMode'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'BluetoothAudioModeOpts'
        QtMocHelpers::PropertyData<QStringList>(16, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'Cards'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'CardsWithoutUnavailable'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'DefaultSink'
        QtMocHelpers::PropertyData<QDBusObjectPath>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'DefaultSource'
        QtMocHelpers::PropertyData<QDBusObjectPath>(21, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'IncreaseVolume'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'MaxUIVolume'
        QtMocHelpers::PropertyData<double>(23, QMetaType::Double, QMC::DefaultPropertyFlags),
        // property 'ReduceNoise'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'SinkInputs'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Sinks'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(27, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Sources'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(28, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<voicevolumewatcher_interface, qt_meta_tag_ZN28voicevolumewatcher_interfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject voicevolumewatcher_interface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28voicevolumewatcher_interfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28voicevolumewatcher_interfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28voicevolumewatcher_interfaceE_t>.metaTypes,
    nullptr
} };

void voicevolumewatcher_interface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<voicevolumewatcher_interface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->PortEnabledChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 1: { QDBusPendingReply<bool> _r = _t->IsPortEnabled((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<> _r = _t->Reset();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<> _r = _t->SetBluetoothAudioMode((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<> _r = _t->SetDefaultSink((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->SetDefaultSource((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->SetPort((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->SetPortEnabled((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (voicevolumewatcher_interface::*)(uint , const QString & , bool )>(_a, &voicevolumewatcher_interface::PortEnabledChanged, 0))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
        case 11:
        case 10:
        case 9:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QDBusObjectPath> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->bluetoothAudioMode(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->bluetoothAudioModeOpts(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->cards(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->cardsWithoutUnavailable(); break;
        case 4: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->defaultSink(); break;
        case 5: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->defaultSource(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->increaseVolume(); break;
        case 7: *reinterpret_cast<double*>(_v) = _t->maxUIVolume(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->reduceNoise(); break;
        case 9: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->sinkInputs(); break;
        case 10: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->sinks(); break;
        case 11: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->sources(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 6: _t->setIncreaseVolume(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setReduceNoise(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *voicevolumewatcher_interface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *voicevolumewatcher_interface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28voicevolumewatcher_interfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int voicevolumewatcher_interface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void voicevolumewatcher_interface::PortEnabledChanged(uint _t1, const QString & _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
