/****************************************************************************
** Meta object code from reading C++ file 'event_monitor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "event_monitor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'event_monitor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12EventMonitorE_t {};
} // unnamed namespace

template <> constexpr inline auto EventMonitor::qt_create_metaobjectdata<qt_meta_tag_ZN12EventMonitorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "EventMonitor",
        "mouseDrag",
        "",
        "x",
        "y",
        "mousePress",
        "mouseRelease",
        "mouseMove",
        "mouseScroll",
        "mouseTime",
        "direction",
        "keyboardPress",
        "keyCode",
        "keyboardRelease",
        "activateWindow",
        "ButtonPressEvent",
        "type",
        "str",
        "ButtonReleaseEvent",
        "CursorMoveEvent"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'mouseDrag'
        QtMocHelpers::SignalData<void(int, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'mousePress'
        QtMocHelpers::SignalData<void(int, int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'mouseRelease'
        QtMocHelpers::SignalData<void(int, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'mouseMove'
        QtMocHelpers::SignalData<void(int, int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'mouseScroll'
        QtMocHelpers::SignalData<void(int, int, int, int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { QMetaType::Int, 10 }, { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'keyboardPress'
        QtMocHelpers::SignalData<void(unsigned char)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 12 },
        }}),
        // Signal 'keyboardRelease'
        QtMocHelpers::SignalData<void(unsigned char)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 12 },
        }}),
        // Signal 'activateWindow'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ButtonPressEvent'
        QtMocHelpers::SlotData<void(int, int, int, QString)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { QMetaType::Int, 3 }, { QMetaType::Int, 4 }, { QMetaType::QString, 17 },
        }}),
        // Slot 'ButtonReleaseEvent'
        QtMocHelpers::SlotData<void(int, int, int, QString)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { QMetaType::Int, 3 }, { QMetaType::Int, 4 }, { QMetaType::QString, 17 },
        }}),
        // Slot 'CursorMoveEvent'
        QtMocHelpers::SlotData<void(int, int, QString)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 }, { QMetaType::QString, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EventMonitor, qt_meta_tag_ZN12EventMonitorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject EventMonitor::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12EventMonitorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12EventMonitorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12EventMonitorE_t>.metaTypes,
    nullptr
} };

void EventMonitor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EventMonitor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->mouseDrag((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->mousePress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->mouseRelease((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->mouseMove((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->mouseScroll((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 5: _t->keyboardPress((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1]))); break;
        case 6: _t->keyboardRelease((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1]))); break;
        case 7: _t->activateWindow(); break;
        case 8: _t->ButtonPressEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 9: _t->ButtonReleaseEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 10: _t->CursorMoveEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (EventMonitor::*)(int , int )>(_a, &EventMonitor::mouseDrag, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (EventMonitor::*)(int , int )>(_a, &EventMonitor::mousePress, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (EventMonitor::*)(int , int )>(_a, &EventMonitor::mouseRelease, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (EventMonitor::*)(int , int )>(_a, &EventMonitor::mouseMove, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (EventMonitor::*)(int , int , int , int )>(_a, &EventMonitor::mouseScroll, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (EventMonitor::*)(unsigned char )>(_a, &EventMonitor::keyboardPress, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (EventMonitor::*)(unsigned char )>(_a, &EventMonitor::keyboardRelease, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (EventMonitor::*)()>(_a, &EventMonitor::activateWindow, 7))
            return;
    }
}

const QMetaObject *EventMonitor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EventMonitor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12EventMonitorE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int EventMonitor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void EventMonitor::mouseDrag(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void EventMonitor::mousePress(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void EventMonitor::mouseRelease(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void EventMonitor::mouseMove(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void EventMonitor::mouseScroll(int _t1, int _t2, int _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 5
void EventMonitor::keyboardPress(unsigned char _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void EventMonitor::keyboardRelease(unsigned char _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void EventMonitor::activateWindow()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
