/****************************************************************************
** Meta object code from reading C++ file 'scrollScreenshot.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "utils/scrollScreenshot.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scrollScreenshot.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16ScrollScreenshotE_t {};
} // unnamed namespace

template <> constexpr inline auto ScrollScreenshot::qt_create_metaobjectdata<qt_meta_tag_ZN16ScrollScreenshotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ScrollScreenshot",
        "getOneImg",
        "",
        "updatePreviewImg",
        "QImage",
        "img",
        "merageError",
        "PixMergeThread::MergeErrorValue",
        "autoScroll",
        "autoScrollFlag",
        "sigalWheelScrolling",
        "direction",
        "merageImgState",
        "state",
        "merageInvalidArea",
        "QRect",
        "rect"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'getOneImg'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updatePreviewImg'
        QtMocHelpers::SignalData<void(QImage)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'merageError'
        QtMocHelpers::SignalData<void(PixMergeThread::MergeErrorValue)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'autoScroll'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Signal 'sigalWheelScrolling'
        QtMocHelpers::SignalData<void(float)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 11 },
        }}),
        // Slot 'merageImgState'
        QtMocHelpers::SlotData<void(PixMergeThread::MergeErrorValue)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 13 },
        }}),
        // Slot 'merageInvalidArea'
        QtMocHelpers::SlotData<void(PixMergeThread::MergeErrorValue, QRect)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 13 }, { 0x80000000 | 15, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ScrollScreenshot, qt_meta_tag_ZN16ScrollScreenshotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ScrollScreenshot::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ScrollScreenshotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ScrollScreenshotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16ScrollScreenshotE_t>.metaTypes,
    nullptr
} };

void ScrollScreenshot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScrollScreenshot *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->getOneImg(); break;
        case 1: _t->updatePreviewImg((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 2: _t->merageError((*reinterpret_cast<std::add_pointer_t<PixMergeThread::MergeErrorValue>>(_a[1]))); break;
        case 3: _t->autoScroll((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->sigalWheelScrolling((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 5: _t->merageImgState((*reinterpret_cast<std::add_pointer_t<PixMergeThread::MergeErrorValue>>(_a[1]))); break;
        case 6: _t->merageInvalidArea((*reinterpret_cast<std::add_pointer_t<PixMergeThread::MergeErrorValue>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QRect>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PixMergeThread::MergeErrorValue >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PixMergeThread::MergeErrorValue >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PixMergeThread::MergeErrorValue >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ScrollScreenshot::*)()>(_a, &ScrollScreenshot::getOneImg, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScrollScreenshot::*)(QImage )>(_a, &ScrollScreenshot::updatePreviewImg, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScrollScreenshot::*)(PixMergeThread::MergeErrorValue )>(_a, &ScrollScreenshot::merageError, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScrollScreenshot::*)(int )>(_a, &ScrollScreenshot::autoScroll, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScrollScreenshot::*)(float )>(_a, &ScrollScreenshot::sigalWheelScrolling, 4))
            return;
    }
}

const QMetaObject *ScrollScreenshot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScrollScreenshot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ScrollScreenshotE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ScrollScreenshot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void ScrollScreenshot::getOneImg()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ScrollScreenshot::updatePreviewImg(QImage _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ScrollScreenshot::merageError(PixMergeThread::MergeErrorValue _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ScrollScreenshot::autoScroll(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void ScrollScreenshot::sigalWheelScrolling(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
