/****************************************************************************
** Meta object code from reading C++ file 'main_window.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "main_window.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'main_window.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainWindow",
        "releaseEvent",
        "",
        "saveActionTriggered",
        "unDo",
        "isInUndoBtn",
        "isInUndo",
        "unDoAll",
        "deleteShapes",
        "stopRecordArm",
        "setUndoEnable",
        "status",
        "microPhoneEnable",
        "saveClipboardComing",
        "onExit",
        "startRecord",
        "shotCurrentImg",
        "addCursorToImage",
        "shotFullScreen",
        "isFull",
        "onHelp",
        "updateCaptureRegion",
        "stopRecord",
        "stopApp",
        "startCountdown",
        "showPressFeedback",
        "x",
        "y",
        "showDragFeedback",
        "showReleaseFeedback",
        "responseEsc",
        "compositeChanged",
        "updateToolBarPos",
        "updateSideBarPos",
        "updateCameraWidgetPos",
        "getTwoScreenIntersectPos",
        "QPoint",
        "rawPos",
        "changeFunctionButton",
        "type",
        "showKeyBoardButtons",
        "key",
        "changeKeyBoardShowEvent",
        "checked",
        "changeMouseShowEvent",
        "changeCameraSelectEvent",
        "handleOptionMenuShown",
        "updateMultiKeyBoardPos",
        "changeShotToolEvent",
        "func",
        "saveScreenShot",
        "saveAction",
        "QPixmap",
        "pix",
        "sendNotify",
        "SaveAction",
        "saveFilePath",
        "succeed",
        "reloadImage",
        "effect",
        "radius",
        "shotImgWidthEffect",
        "onViewShortcut",
        "shapeClickedSlot",
        "shape",
        "on_CheckVideoCouldUse",
        "canUse",
        "stopRecordResource",
        "exitApp",
        "confirm",
        "shotKeyPressEvent",
        "keyCode",
        "recordKeyPressEvent",
        "tableRecordSet",
        "onActivateWindow",
        "onMouseDrag",
        "onMousePress",
        "onMouseRelease",
        "onMouseMove",
        "onMouseScroll",
        "mouseTime",
        "direction",
        "onKeyboardPressWayland",
        "onKeyboardReleaseWayland",
        "onKeyboardPress",
        "onKeyboardRelease",
        "scrollShotMouseClickEvent",
        "scrollShotMouseMoveEvent",
        "scrollShotMouseScrollEvent",
        "onScrollShotCheckScrollType",
        "autoScrollFlag",
        "onLockScreenEvent",
        "QDBusMessage",
        "msg",
        "onSaveClipboardComing",
        "onPowersource",
        "flag",
        "onOpenScrollShotHelp",
        "onAdjustCaptureArea",
        "onScrollShotMerageImgState",
        "PixMergeThread::MergeErrorValue",
        "state",
        "initPadShot",
        "exitScreenCuptureEvent",
        "showPreviewWidgetImage",
        "QImage",
        "img",
        "onExitScreenCapture",
        "onScreenResolutionChanged",
        "onSourceFailed",
        "uint32_t",
        "reason"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'releaseEvent'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'saveActionTriggered'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'unDo'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isInUndoBtn'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Signal 'unDoAll'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'deleteShapes'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stopRecordArm'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'setUndoEnable'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'microPhoneEnable'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'saveClipboardComing'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onExit'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startRecord'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'shotCurrentImg'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addCursorToImage'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'shotFullScreen'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Slot 'shotFullScreen'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onHelp'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateCaptureRegion'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stopRecord'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'stopApp'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'startCountdown'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showPressFeedback'
        QtMocHelpers::SlotData<void(int, int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'showDragFeedback'
        QtMocHelpers::SlotData<void(int, int)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'showReleaseFeedback'
        QtMocHelpers::SlotData<void(int, int)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'responseEsc'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'compositeChanged'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateToolBarPos'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateSideBarPos'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateCameraWidgetPos'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getTwoScreenIntersectPos'
        QtMocHelpers::SlotData<QPoint(QPoint)>(35, 2, QMC::AccessPublic, 0x80000000 | 36, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'changeFunctionButton'
        QtMocHelpers::SlotData<void(QString)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 39 },
        }}),
        // Slot 'showKeyBoardButtons'
        QtMocHelpers::SlotData<void(const QString &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 41 },
        }}),
        // Slot 'changeKeyBoardShowEvent'
        QtMocHelpers::SlotData<void(bool)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 43 },
        }}),
        // Slot 'changeMouseShowEvent'
        QtMocHelpers::SlotData<void(bool)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 43 },
        }}),
        // Slot 'changeCameraSelectEvent'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 43 },
        }}),
        // Slot 'handleOptionMenuShown'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateMultiKeyBoardPos'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'changeShotToolEvent'
        QtMocHelpers::SlotData<void(const QString &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 49 },
        }}),
        // Slot 'saveScreenShot'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveAction'
        QtMocHelpers::SlotData<bool(const QPixmap &)>(51, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 52, 53 },
        }}),
        // Slot 'sendNotify'
        QtMocHelpers::SlotData<void(SaveAction, QString, const bool)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 51 }, { QMetaType::QString, 56 }, { QMetaType::Bool, 57 },
        }}),
        // Slot 'reloadImage'
        QtMocHelpers::SlotData<void(QString, int)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 59 }, { QMetaType::Int, 60 },
        }}),
        // Slot 'shotImgWidthEffect'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onViewShortcut'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'shapeClickedSlot'
        QtMocHelpers::SlotData<void(QString)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Slot 'on_CheckVideoCouldUse'
        QtMocHelpers::SlotData<void(bool)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 66 },
        }}),
        // Slot 'stopRecordResource'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'exitApp'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'confirm'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'shotKeyPressEvent'
        QtMocHelpers::SlotData<void(const unsigned char &)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 71 },
        }}),
        // Slot 'recordKeyPressEvent'
        QtMocHelpers::SlotData<void(const unsigned char &)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 71 },
        }}),
        // Slot 'tableRecordSet'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onActivateWindow'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onMouseDrag'
        QtMocHelpers::SlotData<void(int, int)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'onMousePress'
        QtMocHelpers::SlotData<void(int, int)>(76, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'onMouseRelease'
        QtMocHelpers::SlotData<void(int, int)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'onMouseMove'
        QtMocHelpers::SlotData<void(int, int)>(78, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'onMouseScroll'
        QtMocHelpers::SlotData<void(int, int, int, int)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 80 }, { QMetaType::Int, 81 }, { QMetaType::Int, 26 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'onKeyboardPressWayland'
        QtMocHelpers::SlotData<void(const int)>(82, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 41 },
        }}),
        // Slot 'onKeyboardReleaseWayland'
        QtMocHelpers::SlotData<void(const int)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 41 },
        }}),
        // Slot 'onKeyboardPress'
        QtMocHelpers::SlotData<void(unsigned char)>(84, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 71 },
        }}),
        // Slot 'onKeyboardRelease'
        QtMocHelpers::SlotData<void(unsigned char)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 71 },
        }}),
        // Slot 'scrollShotMouseClickEvent'
        QtMocHelpers::SlotData<void(int, int)>(86, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'scrollShotMouseMoveEvent'
        QtMocHelpers::SlotData<void(int, int)>(87, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'scrollShotMouseScrollEvent'
        QtMocHelpers::SlotData<void(int, int, int, int)>(88, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 80 }, { QMetaType::Int, 81 }, { QMetaType::Int, 26 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'onScrollShotCheckScrollType'
        QtMocHelpers::SlotData<void(int)>(89, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 90 },
        }}),
        // Slot 'onLockScreenEvent'
        QtMocHelpers::SlotData<void(QDBusMessage)>(91, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 92, 93 },
        }}),
        // Slot 'onSaveClipboardComing'
        QtMocHelpers::SlotData<void(const QByteArray &)>(94, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 93 },
        }}),
        // Slot 'onPowersource'
        QtMocHelpers::SlotData<void(bool)>(95, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 96 },
        }}),
        // Slot 'onOpenScrollShotHelp'
        QtMocHelpers::SlotData<void()>(97, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onAdjustCaptureArea'
        QtMocHelpers::SlotData<void()>(98, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onScrollShotMerageImgState'
        QtMocHelpers::SlotData<void(PixMergeThread::MergeErrorValue)>(99, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 100, 101 },
        }}),
        // Slot 'initPadShot'
        QtMocHelpers::SlotData<void()>(102, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'exitScreenCuptureEvent'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showPreviewWidgetImage'
        QtMocHelpers::SlotData<void(QImage)>(104, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 105, 106 },
        }}),
        // Slot 'onExitScreenCapture'
        QtMocHelpers::SlotData<void()>(107, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onScreenResolutionChanged'
        QtMocHelpers::SlotData<void()>(108, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onSourceFailed'
        QtMocHelpers::SlotData<void(uint32_t)>(109, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 110, 111 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MainWindowE_t>.metaTypes,
    nullptr
} };

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->releaseEvent(); break;
        case 1: _t->saveActionTriggered(); break;
        case 2: _t->unDo(); break;
        case 3: _t->isInUndoBtn((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->unDoAll(); break;
        case 5: _t->deleteShapes(); break;
        case 6: _t->stopRecordArm(); break;
        case 7: _t->setUndoEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->microPhoneEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->saveClipboardComing(); break;
        case 10: _t->onExit(); break;
        case 11: _t->startRecord(); break;
        case 12: _t->shotCurrentImg(); break;
        case 13: _t->addCursorToImage(); break;
        case 14: _t->shotFullScreen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->shotFullScreen(); break;
        case 16: _t->onHelp(); break;
        case 17: _t->updateCaptureRegion(); break;
        case 18: _t->stopRecord(); break;
        case 19: _t->stopApp(); break;
        case 20: _t->startCountdown(); break;
        case 21: _t->showPressFeedback((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 22: _t->showDragFeedback((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 23: _t->showReleaseFeedback((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 24: _t->responseEsc(); break;
        case 25: _t->compositeChanged(); break;
        case 26: _t->updateToolBarPos(); break;
        case 27: _t->updateSideBarPos(); break;
        case 28: _t->updateCameraWidgetPos(); break;
        case 29: { QPoint _r = _t->getTwoScreenIntersectPos((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QPoint*>(_a[0]) = std::move(_r); }  break;
        case 30: _t->changeFunctionButton((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 31: _t->showKeyBoardButtons((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 32: _t->changeKeyBoardShowEvent((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->changeMouseShowEvent((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->changeCameraSelectEvent((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: _t->handleOptionMenuShown(); break;
        case 36: _t->updateMultiKeyBoardPos(); break;
        case 37: _t->changeShotToolEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 38: _t->saveScreenShot(); break;
        case 39: { bool _r = _t->saveAction((*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 40: _t->sendNotify((*reinterpret_cast<std::add_pointer_t<SaveAction>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 41: _t->reloadImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 42: _t->shotImgWidthEffect(); break;
        case 43: _t->onViewShortcut(); break;
        case 44: _t->shapeClickedSlot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 45: _t->on_CheckVideoCouldUse((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 46: _t->stopRecordResource(); break;
        case 47: _t->exitApp(); break;
        case 48: _t->confirm(); break;
        case 49: _t->shotKeyPressEvent((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1]))); break;
        case 50: _t->recordKeyPressEvent((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1]))); break;
        case 51: _t->tableRecordSet(); break;
        case 52: _t->onActivateWindow(); break;
        case 53: _t->onMouseDrag((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 54: _t->onMousePress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 55: _t->onMouseRelease((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 56: _t->onMouseMove((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 57: _t->onMouseScroll((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 58: _t->onKeyboardPressWayland((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 59: _t->onKeyboardReleaseWayland((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 60: _t->onKeyboardPress((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1]))); break;
        case 61: _t->onKeyboardRelease((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1]))); break;
        case 62: _t->scrollShotMouseClickEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 63: _t->scrollShotMouseMoveEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 64: _t->scrollShotMouseScrollEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 65: _t->onScrollShotCheckScrollType((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 66: _t->onLockScreenEvent((*reinterpret_cast<std::add_pointer_t<QDBusMessage>>(_a[1]))); break;
        case 67: _t->onSaveClipboardComing((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 68: _t->onPowersource((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 69: _t->onOpenScrollShotHelp(); break;
        case 70: _t->onAdjustCaptureArea(); break;
        case 71: _t->onScrollShotMerageImgState((*reinterpret_cast<std::add_pointer_t<PixMergeThread::MergeErrorValue>>(_a[1]))); break;
        case 72: _t->initPadShot(); break;
        case 73: _t->exitScreenCuptureEvent(); break;
        case 74: _t->showPreviewWidgetImage((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 75: _t->onExitScreenCapture(); break;
        case 76: _t->onScreenResolutionChanged(); break;
        case 77: _t->onSourceFailed((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 40:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SaveAction >(); break;
            }
            break;
        case 66:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusMessage >(); break;
            }
            break;
        case 71:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PixMergeThread::MergeErrorValue >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::releaseEvent, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::saveActionTriggered, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::unDo, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)(bool )>(_a, &MainWindow::isInUndoBtn, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::unDoAll, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::deleteShapes, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::stopRecordArm, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)(bool )>(_a, &MainWindow::setUndoEnable, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)(bool )>(_a, &MainWindow::microPhoneEnable, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::saveClipboardComing, 9))
            return;
    }
}

const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 78)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 78;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 78)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 78;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::releaseEvent()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MainWindow::saveActionTriggered()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MainWindow::unDo()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MainWindow::isInUndoBtn(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void MainWindow::unDoAll()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MainWindow::deleteShapes()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void MainWindow::stopRecordArm()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void MainWindow::setUndoEnable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void MainWindow::microPhoneEnable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void MainWindow::saveClipboardComing()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
